/*
 * $Id: NullDNSCache.java 249 2005-08-26 18:44:12Z davidb $
 * 
 * Copyright (c) 2005 VeriSign. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. 2. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. 3. The name of the author may not
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package se.rfc.unbound.cache;

import java.util.Properties;

import org.apache.log4j.Logger;

import org.xbill.DNS.Name;
import org.xbill.DNS.Section;
import org.xbill.DNS.Type;

import se.rfc.unbound.Request;
import se.rfc.unbound.SMessage;
import se.rfc.unbound.SRRset;

/**
 * The minimal cache. This cache implements a degenerate cache: one that
 * discards all writes.
 * 
 * Note that this cache is only useful for testing. Use of a resolver with
 * this cache is extremely discouraged. So discouraged that it will probably
 * never be shipped.
 * 
 * @author davidb
 */
public class NullDNSCache implements DNSCache
{

  // NOTE: normally DelegationPoint objects are meant to be ephemeral. In this
  // case, we build a root delegation point and just clone it.
  private SMessage rootReferral;

  private SMessage lastCachedReferral;

  private Logger   log = Logger.getLogger(this.getClass());

  public NullDNSCache()
  {
  }

  /**
   * Initialize the cache.
   */
  public void init(Properties config) throws Exception
  {
    // Nothing to do at the moment.
  }

  /**
   * Find something directly stored in the cache.
   *  
   */
  public SMessage lookup(Name qname, int qtype, int qclass)
  {
    // Since we don't cache anything, we never return anything!
    return null;
  }

  /**
   * Store a message in the cache.
   */
  public void store(SMessage response, Request orignalRequest, boolean isReferral)
  {
    // If the message looks something like a referral, save it.
    SRRset[] rrsets = response.getSectionRRsets(Section.AUTHORITY, Type.NS);
    if (rrsets.length > 0)
    {
      log.debug("storing a referral-like message");
      lastCachedReferral = response;
    }

    if (rrsets[0].getName().equals(Name.root))
    {
      rootReferral = response;
    }
    // Otherwise, since we don't cache anything, we don't have anything to do.
  }

  /**
   * Find the closest delegation point.
   */
  public SMessage findDelegation(Name qname, int qclass)
  {
    if (lastCachedReferral != null)
    {
      SRRset[] rrsets = lastCachedReferral.getSectionRRsets(Section.AUTHORITY, Type.NS);
      if (rrsets.length > 0 && qname.subdomain(rrsets[0].getName()))
      {
        return lastCachedReferral;
      }
    }

    return rootReferral;
  }
}